#include "bigDataLoader.hxx"
#include <string.h>
#include <math.h>

void LoadStations(char* stationsFile, WeatherService*& ws)
{
	//how the data is stored in file
	/*
		station name (char* )
		station abbrv (char* - length 3)
		float longitude
		float latitude
		int   elevation
	*/
	if(stationsFile == NULL)
	{
		fprintf(stderr,"No file name given!\n");
		return;	
	}
	
	FILE* inFile = fopen(stationsFile,"r");
	if( inFile == NULL)
	{
		fprintf(stderr,"Failed to open file %s\n", stationsFile);
		return;
	}
	
	
	Station* temp;
	char buffer[100];
	while( !feof(inFile) )
	{
		temp = new Station();
		//fscanf(inFile,"%s,%s,%f,%f,%d\n",
		//	buffer, temp->stationAbbrv,
		//	&(temp->longitude),&(temp->latitude),
		//	&(temp->elevation));
		
		long startpos;
		long endpos;
		startpos = ftell(inFile);
		while( fgetc(inFile) != ',');
		endpos = ftell(inFile);
		fseek(inFile,startpos,0);
		memset(buffer,0,100);
		fread(buffer,1,endpos - startpos - 1,inFile);
		temp->stationName = new char[ strlen(buffer) + 1];
		strcpy(temp->stationName,buffer);
		fseek(inFile, endpos, 0);
		
		//---------------------------------
		startpos = ftell(inFile);
		while( fgetc(inFile) != ',');
		endpos = ftell(inFile);
		fseek(inFile,startpos,0);
		memset(buffer,0,100);
		fread(buffer,1,endpos - startpos - 1,inFile);
		temp->stationAbbrv = new char[ strlen(buffer) + 1];
		strcpy(temp->stationAbbrv,buffer);
		fseek(inFile, endpos, 0);
				
		//------------------------------
		fscanf(inFile,"%f,%f,%d\n",
			&(temp->latitude),&(temp->longitude),
			&(temp->elevation));
		
		//temp->stationName = new char[strlen(buffer) + 1];
		//strcpy(temp->stationName,buffer);
			
		ws->stationListing.push_back( temp );
		
		printf("-----------------\nloaded following data\nbuffer:%s,abbrv:%s,floats:%f,%f,elev:%d\n",
			temp->stationName,temp->stationAbbrv,
			(temp->latitude),(temp->longitude),
			(temp->elevation));
			
		temp = NULL;	
	}

	fclose(inFile);
}


void LoadStationDailyWeather(char* dailyWeatherFile, long& numberOfDays, DayInFile*& dailyList)
{
	if(dailyWeatherFile == NULL)
	{
		dailyList = NULL;
		fprintf(stderr,"No file name given!\n");
		return;
	}

	numberOfDays = 0;

	//make sure the dailyList is clean
	if(dailyList)
	{
//		fprintf(stderr,"Cleaning out list\n");
		while(dailyList)
		{
			DayInFile* temp = dailyList;
			dailyList = dailyList->next;
			delete temp;
		}


	}

//	fprintf(stderr,"list clean\n");

	dailyList = NULL;

	//open the file
	printf("loading file %s\n",dailyWeatherFile);
	FILE* dailyFile = fopen(dailyWeatherFile,"r");
	if(dailyFile == NULL)
	{
		fprintf(stderr,"Failed to open file %s\n", dailyWeatherFile);
		return;
	}

	long currentPosition;
	long nextPosition;

	//keep going until we start on the third row
	while( fgetc(dailyFile) != '\n' );
	while( fgetc(dailyFile) != '\n' );
	DayInFile* tail = dailyList;
//	fprintf(stderr,"On third row!\n");
//	fprintf(stderr,"Loading Station data for station %s", dailyWeatherFile);
	while( !feof(dailyFile))
	{
		//keep going until we hit a new line
		//when we do then we create a DayInFile struct and add it to the list
		currentPosition = ftell(dailyFile);
//		printf("Storing current position - ");
		//loop to end of row
		char currentChar;
		do
		{
			currentChar = fgetc(dailyFile);
		}
		while( currentChar != '\n' && currentChar != '\n' && !feof(dailyFile));
//		printf("Found end of the line\n");
		nextPosition = ftell(dailyFile);

		//grab the info
		DayInFile* newDay = new DayInFile();
		newDay->fileOffset = currentPosition;
		fseek(dailyFile, currentPosition, 0);
		//get the year, month, day
//		printf("getting year, month, day:");
		fscanf(dailyFile,"%d,%d,%d", &(newDay->year), &(newDay->month), &(newDay->day) );
//		printf("getting averages!\n");
		float fields[19];
		char error;
//		fprintf(stderr,"Fields: ");
		for(int fieldCount = 0; fieldCount < 19; fieldCount++)
		{
			fscanf(dailyFile,",%f,%c",&fields[fieldCount],&error);
			if(error == ',')
			{	
				long pos = ftell(dailyFile);
				fseek(dailyFile, pos - 1, 0);
				
			}
			
//			fprintf(stderr,"f: %f, err: %c ", fields[fieldCount],error);
		}
//		fprintf(stderr,".");
		
		//now get the averages we want
		newDay->avgWindSpeed = fields[1];
		newDay->avgWindDir = fields[2];
		newDay->avgAirTemp = fields[6];
		newDay->avgHumidity = fields[9];
		newDay->totalPrecip = fields[11];
		newDay->avgSoilTemp = fields[15];
		fseek(dailyFile, nextPosition, 0);

		//if for some reason we have values 0,0,0 then something funny happened and
		//we must be past the end of file....next couple of lines is a hack to fix that problem
		if(newDay->year == 0 || newDay->month == 0 || newDay->day == 0)
			break;

//		printf(" got it! %d, %d, %d\nmoving file pointer to end of row\n", newDay->year, newDay->month, newDay->day);

		//add the object to the end of the list;
//		printf("adding object to end of list\n");
		if( dailyList == NULL )
		{
//			printf("first node!\n");
			dailyList = newDay;
			newDay->next = NULL;
			tail = dailyList;
		}
		else
		{
			tail->next = newDay;
			newDay->next = NULL;
			tail = newDay;
		}
//		printf("copying address of FILE object");
		newDay->dataFile = dailyFile;
//		printf("moving on to address %d\n", nextPosition);
//		fprintf(stderr,"adding to row count: %d\n", numberOfDays + 1);
		numberOfDays++;	//another day has been added


	}	//end looping to end of file

	fclose(dailyFile);


}


void DeleteStations(WeatherService*& ws)
{
	if(ws == NULL)
		return;

	Station* s = NULL;
	int station = 0;
	while(station < ws->stationListing.size() )
	{
		s = ws->stationListing[station];
		
		if( !s )
			continue;
		DayInFile* temp;
		
		while(s->dailyList)
		{
			temp = s->dailyList;
	//		printf("deleting %d,%d,%d\n", temp->year, temp->month, temp->day);
			s->dailyList = s->dailyList->next;
			delete temp;
		}
		
		if( s->stationName ) delete [] s->stationName;
		if( s->stationAbbrv) delete [] s->stationAbbrv;
		
		delete s;
		s = NULL;
		station++;
	}
	
	ws->stationListing.clear();
}

DayInFile* GetDayInFileArray(int month, int day, int year,WeatherService* ws,int& returnLength)
{
	if( ws == NULL)
	{
		fprintf(stderr,"Recieved NULL WeatherService object...returning NULL\n");
		return NULL;
	}
	
	DayInFile* days = new DayInFile[ws->stationListing.size()];
	Station* s = NULL;
	int station = 0;
	while(station < ws->stationListing.size() )
	{
		s = ws->stationListing[station];
		
		if( !s )
			continue;
		DayInFile* temp = s->dailyList;
		
		while(temp )
		{
			if(temp->month == month && temp->day == day && temp->year == year)
				break;
			temp = temp->next;
		}
		
		if(temp)
		{
			memcpy(&days[station],temp,sizeof(DayInFile));
			days[station].dataFile = NULL;
			days[station].next = NULL;
		}
		else
		{
			fprintf(stderr,"Nonexistant Day at Station %s\n",s->stationName);
			delete [] days;
			return NULL;
		}
		station++;
	}
	
	returnLength = station;
	return days;
}

void GetNextDate(int& month, int& day, int& year)
{
	bool leapYear = false;
	if( year % 4 == 0)
		leapYear = true;

	int lastDay = 31;
	switch(month)
	{
	case 2:
		if(leapYear)
			lastDay = 29;
		else
			lastDay = 28;
		break;
	case 4:
	case 6:
	case 9:
	case 11:
		lastDay = 30;
		break;
	default:
		//nothing to do
		break;
	}
	
	if( ++day > lastDay )
	{
		day = 1;
		month++;
		if( month > 12)
		{
			month = 1;
			year++;
		}
	}
}
	
double Distance(double x1, double y1, double x2, double y2)
{
//	cerr << "* ";
//	cerr << x2 << ' ' << x1 << ' ' << x2 - x1 << "* ";
	x1 = (x2 - x1);
//	cerr << y2 << ' ' << y1 << ' ' << y2 - y1 << "* ";
	y1 = (y2 - y1);
	x1 *= x1;
//	cerr << '*';
	y1 *= y1;
//	cerr << '*';
	x1 += y1;
//	cerr << sqrt(x1);
	return sqrt(x1);

}
