
#include "vtkapp.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#define DEG_RAD_RATIO (3.14159f / 360.0f )
#define DEG_TO_RAD( a ) ( (a) * DEG_RAD_RATIO )
//#include <conio.h>
#define SHEPARD_RESULT_TEST 0
#define SHEPARD_PRINT_SCREEN 0
#define X 0
#define Y 1
#define Z 2
#define W 3

#define RED 0
#define GREEN 1
#define BLUE 2
#define YELLOW 3
#define ALPHA 3

//---------------------------------------
void VTKApp::CreateWindDirectionGlyphs(void* data)
{
	cerr << "Create Wind Direction Glyphs called" << endl;
	VTKAppMessage* msg = (VTKAppMessage* )data;
	InterestedDay* interest = (InterestedDay*) msg->msgData;
	int length = 0;
	float** windValues = NULL; 
	float** speedValues = NULL; 
	int itr;	

	
	
#if SHEPARD_RESULT_TEST
//		FILE* fout = fopen("results.raw","wb");
		FILE* fout2 = fopen("tableMaps.txt","w");
#endif
	
	//OK! The degrees are setup as follows! South is 0 degrees (i.e. a vector pointing 0,-1) and rotation is clockwise!
	//need to adjust the values to get the "proper" 3D graphics rotations and then use shepards algorithm to fix the rest
	cerr << "Setting Number of Days" << endl;
	int numberOfDays = 1;
	if(oneWeekMode)
	{
		numberOfDays = 7;
		if( interest->year == LAST_YEAR && interest->month == 12 && (interest->day + 7) > 31)
		{
			numberOfDays -= (interest->day + 7) - 31; 
		}
	}

	int dayItr = 0;
	int month = interest->month;
	int day = interest->day;
	int year = interest->year;
	int pixelX,pixelY;
	float MinValue = 1000, MaxValue = -1000;

	//--------GENERATE THE WIND_SPEED COLOR MAP------------//
	interest->attribute = AVG_WIND_SPEED;
	CreateColorMapActors( (void*) msg );
	
	//_________NOW WE EDIT IT BY DRAWING LINES_________________//
	
	dayItr = 0;
	cerr << "Starting First Day" << endl;
	while( dayItr < numberOfDays)
	{
		canvases[dayItr]->SetDrawColor( 254 );
	
		
		int length;
		length = 0;
		if(dayItr > 0)
			GetNextDate(month,day,year);
		cerr << "Day of Interest: " << month << ',' << day << ' ' << year << endl;
		
		DayInFile* daysArray = NULL;
		cerr << "Global Weather Service Object at @ " << gWeatherService << " -- WeatherService Sent to Method @: " << interest->pWS << endl;
		WeatherService* ws = gWeatherService;
		
		if( interest->pWS != NULL)
		{
			ws = interest->pWS;
			cerr << "Accessing Weather Service at: " << interest->pWS << endl;
			daysArray = GetDayInFileArray( month, day, year,  interest->pWS, length);
		}
		else	
		{
			cerr << "Accessing Global Weather Service" << endl;
			daysArray = GetDayInFileArray( month, day, year, gWeatherService, length);
		}
		
		cerr << "WIND DIRECTION ATTRIBUTES: " << endl;
		if(windValues)
			delete [] windValues;
		if( length == 0)
			return;
			
		windValues = new float*[length];
		for(int i = 0; i < length; i++)
		{
			windValues[i] = &(daysArray[i].avgWindDir);
			
			cerr << *windValues[i] << ' ' ;
			if( (i + 1) % 5 == 0)
				cerr << endl;
		} 
		
		cerr << "DayArray Address @ " << daysArray << " Number of Scatter Points: " << length << endl;
		
		//get the attributes we are interested in!
		int i;
		
		//-----------------------Begin interpolating and drawing the texture
		cerr << "Starting Inverse Distance Interpolation @ " << clock() << endl;
		//now we actuall run the shepard method!
//		cerr << "dimensions: " << cols << 'x' << rows << " ==> " << cols * rows << " actors " << endl;
		
//		for(pixelY = 0; pixelY < rows; pixelY++)
//		{
//			for(pixelX = 0; pixelX < cols; pixelX++)
//			{
		for(pixelY = 0; pixelY < MAP_VERT_RESOLUTION; pixelY += 25 )
		{
			for(pixelX = 0; pixelX < MAP_HORIZ_RESOLUTION; pixelX += 20 )
			{
#if SHEPARD_PRINT_SCREEN
				cerr << "pixel " << pixelX << ", " << pixelY << ' ';
				
#endif
				double* distance = NULL;
				double maxDistance;
				double pixelLongitude, pixelLatitude;
					
				pixelLongitude = ((double)pixelX * (double)DEG_LONG_PER_DX / 1 )+ (double)DEG_LONG_AT_ZERO;
				pixelLatitude = ((double)pixelY * (double)DEG_LAT_PER_DY / 1 )+ (double)DEG_LAT_AT_ZERO;
				maxDistance = 0;
				int k;
#if SHEPARD_PRINT_SCREEN
				cerr << " pixel " << pixelLongitude << "," << pixelLatitude << " Getting Max Distance - ";
#endif
				//get distance to farthest scatter point
				if(!distance) 
					distance = new double[length];
		//		cerr << "Getting Max Distance: ";
				for( k = 0; k < length; k++)
				{	
#if SHEPARD_PRINT_SCREEN
		//			cerr << endl <<  "scatter point " << ws->stationListing[k]->longitude << ',' << ws->stationListing[k]->latitude << ' ';
#endif
#if SHEPARD_RESULT_TEST
					fprintf(fout2,"scatter point %f, %f -- ",  ws->stationListing[k]->longitude, ws->stationListing[k]->latitude);
					fprintf(fout2,"table point: %f, %f ---\n", pixelLongitude, pixelLatitude);
#endif
		//			cerr << '.' ;
					distance[k] = Distance(pixelLongitude,pixelLatitude,ws->stationListing[k]->longitude,
								ws->stationListing[k]->latitude);
#if SHEPARD_PRINT_SCREEN
			//		cerr << '?';
#endif
					if(distance[k] > maxDistance )
						maxDistance = distance[k];
					
				}
#if SHEPARD_PRINT_SCREEN
				cerr << endl << "Applying weights to value - ";
#endif
				//apply the weights to each attribute value!
				double value;
				double numerator, denominator;
				value = 0;
				numerator = 0;
				denominator = 0;
				
				//______________________GET THE WIND DIRECTION_____________//
				for( k = 0; k < length; k++)
				{
#if SHEPARD_PRINT_SCREEN
	//				cerr << "attribute value: " << *(windValues[k]) << " - distance: " << distance[k] << endl;
#endif

					if((*windValues[k]) < 999.9)
					{
						//value += (weights[k]) * (*(attributeValues[k]));
						numerator += ((*windValues[k]) / (distance[k] * distance[k] * distance[k]));		
						denominator += (1.0f / (distance[k] * distance[k] * distance[k]));
					}
				}
				
				//value = (numerator / denominator);
				float angle;
				value = numerator / denominator;
				angle = (180 - value) + 90;

				
#if SHEPARD_RESULT_TEST
				//fprintf(fout,"%d %d %d ",lookup.tableValues[k][0],lookup.tableValues[k][1],lookup.tableValues[k][2]);
				//cerr << "pixel " << pixelX << ", " << pixelY << endl;
//				fwrite(lookup.tableValues[k],sizeof(unsigned char),3,fout);


				fprintf(fout2,"%d,%d : value: %f @ %d ", pixelX, pixelY, angle, &angle);
					//windDirections[dayItr][pixelX + (pixelY * cols)], &(windDirections[dayItr][pixelX + (pixelY * cols)]) );
					
				fprintf(fout2,"\n");
#endif
			//	cerr << " contiuing "<< endl;
				
				//----------------DRAW THE LINE IN THE DIRECTION OF THE WIND!!!------------//
				//need to convert angle to slope
				//assuming pixel length of about 5
				float dx,dy,negdx,negdy;
				dx = (float)(10.0f * cos( DEG_TO_RAD(angle) ));
				dy = (float)(10.0f * sin( DEG_TO_RAD(angle) ));
				
			//	canvases[dayItr]->DrawSegment( pixelX, pixelY, (int)(pixelX + dx), (int)(pixelY + dy) );
				canvases[dayItr]->FillTriangle( (int)(pixelX + dy / 4), (int)(pixelY - dx / 4), (int)(pixelX + dx), (int)(pixelY + dy), 
								(int)(pixelX - dy / 4), (int)(pixelY + dx / 4));
			//	canvases[dayItr]->FillTriangle( pixelX, pixelY, (int)(pixelX + dx), (int)(pixelY + dy), 
			//					(int)(pixelX ), (int)(pixelY + dy / 3));
								
			}//--------------------End PixelX loop
#if SHEPARD_RESULT_TEST
		fprintf(fout2,"\n");
#endif
			cerr << ".";
			
			
		}//------------------End PixelY loop
		cerr << " --- Finished @ " << clock() << endl;
	
		dayItr++;
	}//---------------------------End While Loop for each day to process the values of that day
#if SHEPARD_RESULT_TEST
	fclose(fout2);
#endif
/*
	cerr << "Adding Actors" << endl;
	//add the actors of the first day
	render1->AddActor( arrowActors[0] );
	for( pixelY = 0; pixelY < rows; pixelY++)
	{
		for(pixelX = 0; pixelX < cols; pixelX++)
		{
			if( arrowActors[pixelY * cols + pixelX])
			{
				render1->AddActor( arrowActors[pixelY * cols + pixelX] );
				cerr << pixelY * rows + pixelX << ' ';
			}
		}
	}
*/
	cerr << "Resetting Camera & rendering" << endl;
	render1->ResetCamera();
	Render();
}

